<?php
namespace ID3\Aliquando;

class Admin extends \ID3\Bases\AdminInstance{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        $this->registerHooks([
            'admin_menu' => 'filter_menu',
            'id3/admin/pluginContext' => 'addAdminPluginContext'
        ]);
        
        add_action( 'delete_blog', [$this,'delete_blog'], 10, 2 );

    }

    function delete_blog( $blog_id, $drop ) {

        if ( true == $drop ) {
    
            /**
             * SELECT all tables relating to a specific blog id and add them to wpmu_drop_tables
             */
            global $wpdb;
            $prep_query = $wpdb->prepare( "SELECT table_name FROM information_schema.TABLES WHERE table_name LIKE %s;", $wpdb->esc_like( "{$wpdb->base_prefix}{$blog_id}_" ) . '%' );
            $table_list = $wpdb->get_results( $prep_query, ARRAY_A );
    
            add_filter( 'wpmu_drop_tables', function ( $filter_list ) use ( $table_list ) {
    
                foreach( $table_list as $index => $data ) {
                    $filter_list[] = $data['table_name'];
                }
    
                return array_unique( $filter_list );
    
            });
        }
    }

    function filter_menu(){
        if(!is_super_admin()){
            $this->loadSettings();

            if($this->networkSettings->block->wpThemes){
                $customizer_url = add_query_arg( 'return', urlencode( remove_query_arg( wp_removable_query_args(), wp_unslash( $_SERVER['REQUEST_URI'] ) ) ), 'customize.php' );
                remove_submenu_page( 'themes.php', $customizer_url );
                remove_submenu_page( 'themes.php', 'themes.php' );
            }

            if($this->networkSettings->block->wpSettings){
                remove_menu_page('options-general.php');
            }

            if($this->networkSettings->block->wpPlugins){
                remove_menu_page('plugins.php');
            }
        }
    }

    function addAdminPluginContext($data){
        $data['currentBlogId'] = get_current_blog_id();
        $data['networkDomainName'] = trim(str_replace(['http://','https://'],'', network_site_url()),'/');
        return $data;
    }

    static private $thirdPartyRequirements = [];
    static function addRequirement($path, $defaults=[]){
        if($path === '') return;
        
        // modify path to fit absolute url format
        // replace 1) remove abspath 2) switch backslash with slash (on Windows server only)
        $path = str_replace([trim(\ABSPATH,'/'),'\\'], ['','/'], $path);
        $path = str_replace('//','/', $path);

        self::$thirdPartyRequirements[] = [
            'path' => $path,
            'defaults' => $defaults
        ];
    }


    static function getThirdPartyRequirements(){
        // Call action to build new site requirement

        do_action('id3-aliquando/requirement/add');

        return self::$thirdPartyRequirements;
    }


    static private $thirdPartyOptionals = [];
    static function addOptional($path, $defaults=[]){
        // modify path to fit absolute url format
        // replace 1) remove abspath 2) switch backslash with slash (on Windows server only)
        $path = str_replace([trim(\ABSPATH,'/'),'\\'], ['','/'], $path);
        $path = str_replace('//','/', $path);
        self::$thirdPartyOptionals[] = [
            'path' => $path,
            'defaults' => $defaults
        ];
    }

    static function getThirdPartyOptionals(){
        // Call action to build new site requirement

        do_action('id3-aliquando/optional/add');

        return self::$thirdPartyOptionals;
    }
}

new Admin();