<?php
namespace ID3\Aliquando;

use ID3\Aliquando\SiteSync;

class Rest extends \ID3\Bases\Rest{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        $this->addRoute('/theme/list', [$this, 'getThemeList']);
        
        

        $this->addRoute('/third-party/requirements', [$this, 'getThirdPartyRequirements'],[
            'args' => [
                'theme' => [
                    'required' => false,
                    'type' => 'string',
                    'description' => 'Template name'
                ]
            ]
        ]);

        $this->addRoute('/third-party/optionals', [$this, 'getThirdPartyOptionals'],[
            'args' => [
                'theme' => [
                    'required' => false,
                    'type' => 'string',
                    'description' => 'Template name'
                ]
            ]
        ]);

        $this->addRoute('/domain/exists', [$this, 'domainExists'],[
            'args' => [
                'name' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Domain name to check',
                ]
            ]
        ]);

        
        $this->addRoute('/sites/search', [$this, 'searchSites'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'keyword' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Search keywords',
                ],
                'limit' => [
                    'required' => false,
                    'type' => 'number',
                    'description' => 'Search result limit',
                ]
            ]
        ]);

        $this->addRoute('/sites/get', [$this, 'getSiteInfos'],[
            'args' => [
                'id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Site id',
                ],
                'source_only' => [
                    'required' => false,
                    'type' => 'boolean',
                    'description' => 'Return only site flagged as blueprint'
                ]
            ]
        ]);

        $this->addRoute('/network/getTemplateInfo', [$this, 'getNetworkTemplateInfo'],[
            'args' => [
                'id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Template id',
                ]
            ]
        ]);

        $this->addRoute('/pages/apply-structure', [$this, 'applyPageStructure'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Creation data context',
                ]
            ]
        ]);

        new RestCreation();
    }


    function getThemeList(){
        return [];
    }

    function getThirdPartyRequirements($request){
        $theme   = $request->get_param('theme');
        if(empty($theme)){
            $theme = get_stylesheet();
        }

        $themesRoot = get_theme_root();
        $themeExtensionFilePath = "$themesRoot/$theme/aliquando_extension.php";
        do_action('id3/bootstrap/admin', true);
        
        if(file_exists($themeExtensionFilePath)){    
            include_once $themeExtensionFilePath;
            //return apply_filters('id3/aliquando/requirements', []);
            return \ID3\Aliquando\Admin::getThirdPartyRequirements();
        }

        
        return [];
    }

    function getThirdPartyOptionals($request){
        $theme   = $request->get_param('theme');
        if(empty($theme)){
            $theme = get_stylesheet();
        }
        $themesRoot = get_theme_root();
        $themeExtensionFilePath = "$themesRoot/$theme/aliquando_extension.php";
        

        if(file_exists($themeExtensionFilePath)){
            do_action('id3/bootstrap/admin', true);
            include_once $themeExtensionFilePath;
            return \ID3\Aliquando\Admin::getThirdPartyOptionals();
        }


        
        return [];
    }


    

    function domainExists($request){
        $name   = $request->get_param('name');
        $result = false;
        try {
            $result = domain_exists($name, '') !== null;
        } catch (\Throwable $th) {
            $result = $th;
        }

        return $result;
    }


    
    function getSiteInfos($request){
        $id = $request->get_param('id');

        $site = get_site($id);

        return [
            'id'        => $site->blog_id,
            'title'     => $site->blogname,
            'domain'    => $site->domain
        ];
    }

    function searchSites($request){
        $kwd = $request->get_param('keyword');
        $limit = $request->get_param('limit');
        $source_only = $request->get_param('source_only');

        if(is_multisite() && function_exists('get_sites')){
            if(!isset($limit)) $limit = 20;
            // Load network settings
            $networkSettings = \ID3\Settings::getNetwork(\ID3\Aliquando\KEY);
            $result = [];
            if($source_only===true){
                $result = $networkSettings->available_sources;
            }
            else{
                $queryOptions = [
                    'orderby'=> 'last_updated',
                    'order' => 'DESC'
                ];
                $currentSourceSiteIds = array_map(function($elm){
                    return $elm->id;
                }, $networkSettings->available_sources);

                $result = \get_sites($queryOptions);
                $result = array_map(function($elm) use ($currentSourceSiteIds){
                    //$elm->get_details();
                    return [
                        'id'        => $elm->blog_id,
                        'title'     => $elm->blogname,
                        'domain'    => $elm->domain,
                        'registered' => $elm->registered,
                        'public' => $elm->public,
                        'archived' => $elm->archived,
                        'is_source' => in_array($elm->blog_id,$currentSourceSiteIds)
                    ];
                },$result);

            }
           
            if($kwd != null && $kwd != ''){
                $result = array_filter($result, function($elm) use ($kwd){
                    return strpos(strtolower($elm['title']), strtolower($kwd))!==false;
                });
            }

            $result = array_slice($result,0,$limit);
            
            return $result;
        }
        else{
            return [
                'err' => 'Not a multisite network installation'
            ];
        }
        
    }

    function getNetworkTemplateInfo($request){
        $template_id = $request->get_param('id');

        switch_to_blog(1);
        
        // query
        $item = get_post($template_id);
        $itemMetas = get_post_meta($item->ID, '_elementor_data');
        $itemData = null;
        if(count($itemMetas) > 0){
            $itemData = json_decode($itemMetas[0])[0];
        }

        $lResult = [
            'title' => $item->post_title,
            'data' => $itemData,
            'nonce' => wp_create_nonce()
        ];
        restore_current_blog();
        return $lResult;
    }

    function applyPageStructure($request){
        $data   = json_decode(json_encode($request->get_param('data')));
        

        $menu_exists = wp_get_nav_menu_object( 'Main' );
        $menu_id = ($menu_exists!==false) ? $menu_exists->term_id : wp_create_nav_menu('Main');
        
        // delete all pages
        if(!$data->keep_pages){
            $pages = get_pages(['post_status'=>'publish']);
            foreach ($pages as $page) {
                wp_delete_post($page->ID, true);
            }

            $navItems = wp_get_nav_menu_items($menu_id);
            foreach($navItems as $nav){
                wp_delete_post($nav->ID, true);
            }
        }

        foreach($data->structure_preset->pages as $page){
            // Create page
            $this->createPage($page);
            
            // Page to menu
            if($page->id != false){

                if(isset($page->is_homepage) && $page->is_homepage === true){
                    update_option( 'page_on_front', $page->id );
                    update_option( 'show_on_front', 'page' );
                }

                wp_update_nav_menu_item($menu_id, 0, [
                    'menu-item-title' =>  $page->title,
                    'menu-item-classes' => sanitize_title($page->title),
                    'menu-item-url' => get_permalink($page->id), 
                    'menu-item-status' => 'publish'
                ]);
            }
        }

        $theme_navs = get_registered_nav_menus();

        $menuAdded = false;

        foreach ( $theme_navs as $location => $description ) {
            if($menuAdded) continue;
            $menuAdded = true;

            $locations = get_theme_mod('nav_menu_locations');
            $locations[$location] = $menu_id;
            set_theme_mod( 'nav_menu_locations', $locations );
        }

    }

    function createPage(&$pageInfo, $parentPage = null){
        $newPage = [
            'post_type' => 'page',
            'post_title' => $pageInfo->title,
            'post_content' => $pageInfo->content,
            'post_status'   => 'publish',
            'post_parent'   => $parentPage
        ];

        $pageInfo->id = wp_insert_post($newPage);
        
        if($pageInfo->id !== false){
            if(isset($pageInfo->children) && is_array($pageInfo->children)){
                foreach($pageInfo->children as $child){
                    $this->createPage($child,$pageInfo->id);
                }
            }
        }
        
        return $pageInfo->id;
    }
}

class RestCreation extends \ID3\Bases\Rest{
    function __construct(){
        parent::__construct(__NAMESPACE__);

        $this->addRoute('/site/create', [$this, 'create'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Creation data context',
                ]
            ]
        ]);
        $this->addRoute('/site/create/sync-content', [$this, 'syncContent'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Creation data context',
                ]
            ]
        ]);
        $this->addRoute('/site/create/sync-files', [$this, 'syncFiles'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Creation data context',
                ]
            ]
        ]);
        $this->addRoute('/site/create/apply-theme', [$this, 'applyTheme'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Creation data context',
                ]
            ]
        ]);
        $this->addRoute('/site/create/finalize', [$this, 'finalize'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Creation data context',
                ]
            ]
        ]);
    }



    
    function create($request){
        $data   = json_decode(json_encode($request->get_param('data')));
        $result = [
            'success' => false,
            'site_id' => false,
            'user_id' => false
        ];

        \ID3\write_log('create');

        //try {
        
            
            $user_id = get_current_user_id();
            
            
            //if(!is_wp_error($result['user_id'])){
                $result['site_id'] = $this->createBlog($data->title,$user_id, $data->domain);

                if(!is_wp_error($result['site_id'])){
                    $result['success'] = true;
                }
                else{
                    $result['site_creation_error'] = $result['site_id'];
                }
            // }
            // else{

            // }

        // } catch (\Throwable $th) {
        //     $result = $this->rollback($result);
        //     $result['rollback_reason'] = $th->getMessage();
        //     new WP_Error(601, $result['rollback_reason']);
        // }
        
        return $result;
    }

    function applyTheme($request){
        $data   = json_decode(json_encode($request->get_param('data')));
        $site_id = $data->site_id;
        $theme = $data->theme;
        $stylesheet = $data->stylesheet;
        $current_theme = $data->current_theme;
        $result = [];
        
        \ID3\write_log('apply theme');

        if(!$this->validateTarget($site_id, $result)){
            $this->throwError($result['error']);
            return $result;
        }

        switch_to_blog($site_id);
        global $wpdb;
        
        $wpdb->query("UPDATE {$wpdb->prefix}options SET option_value='{$theme}' WHERE option_name = 'template'");
        $wpdb->query("UPDATE {$wpdb->prefix}options SET option_value='{$stylesheet}' WHERE option_name = 'stylesheet'");
        $wpdb->query("UPDATE {$wpdb->prefix}options SET option_value='{$current_theme}' WHERE option_name = 'current_theme'");

        $themesRoot = get_theme_root();
        $result['themesRoot'] = $themesRoot;
        $themeExtensionFilePath = "$themesRoot/$stylesheet/aliquando_extension.php";
        $result['themeExtension'] = $themeExtensionFilePath;
        $result['themeExtensionExists'] = file_exists($themeExtensionFilePath);
        
        //return $result;
        try {
            //code...
        
            do_action('id3/bootstrap/admin', true);


            if(file_exists($themeExtensionFilePath)){    
                include_once $themeExtensionFilePath;
                do_action('id3-aliquando/setup/creation', $data->requirements);
            }
        } catch (\Throwable $th) {
            $result['themeExtensionFailed'] = $th->getMessage();
        }

        restore_current_blog();

        return $result;
    }

    function finalize($request){
        $data   = json_decode(json_encode($request->get_param('data')));
        $site_id = $data->site_id;
        $description = isset($data->description) ? $data->description : '';
        $plugins = $data->plugins;
        $result = [];

        \ID3\write_log('finalize');

        if(!$this->validateTarget($site_id, $result)){
            $this->throwError($result['error']);
            return $result;
        }

        switch_to_blog($site_id);
        \ID3\write_log('finalize:switch to site ' . $site_id);

        \update_option('blogdescription', $description);
        \update_option('blog_public', '1');
        

        // add custom logo
        if(isset($data->logo) && !\ID3\isNullOrEmpty($data->logo) ){
            $logoData = base64_decode($data->logo);
            if(false !== $logoData){
                $result['logo_id'] = $this->addCustomLogo($data->title . ' logo', sanitize_title($data->title) . '-logo.png', $logoData);
            }
            else{
                \ID3\write_log('Err: Unable to decode logo data');
            }
            
        }

        $admin_email = '';
        
        // save plugins settings
        foreach($plugins as $item){
            \ID3\Settings::set($item->name,$item->settings);
            if(isset($item->settings->office_email)){
                $admin_email = $item->settings->office_email;
            }
        }
        
        // Add user capabilities
        if($data->create_user === true){
            $data->user_id = $this->createUser($data->user);
            $admin_email = $data->user->email;
            $this->addUserRoleCaps($data->user_id,'administrator');
        }

        if($admin_email != ''){   
            \update_option('admin_email', $admin_email);
            \update_option('new_admin_email', $admin_email);
        }
        // Get Elementor Pro Activation path. ex.: http://test6.wplab.localhost/wp-admin/admin.php?page=elementor-connect&app=activate&action=authorize&nonce=b7d3d251b5
        // $elementorProPath = $data->domain . '/wp-admin/wp-admin/admin.php?page=elementor-connect&app=activate&action=authorize&nonce=' . wp_create_nonce();
        // $result['elementor_activation_path'] = $elementorProPath;
        
        restore_current_blog();
        \ID3\write_log('finalize: Completed');
        return $result;
    }

    function rollback($creationInfos){
        $result = [
            'success' => false
        ];

        // delete site
        if(!is_wp_error($creationInfos['site_id'])){
            wpmu_delete_blog($creationInfos['site_id'], true);
        }
        
        return $result;
    }

    function createUser($userInfos){
        \ID3\write_log('create user');
        $user_name = preg_replace( '/\s+/', '', sanitize_user( $userInfos->name, true ) );
 
        $user_id = wp_create_user( $user_name, $userInfos->password, $userInfos->email );
        if ( is_wp_error( $user_id ) ) {
            $this->throwError('Unable to create user');
            return $user_id;
        }
    
        // Newly created users have no roles or caps until they are added to a blog.
        //delete_user_option( $user_id, 'capabilities' );
        //delete_user_option( $user_id, 'user_level' );

        // Get firstname/lastname
        $nameParts = explode(' ', $userInfos->name);

        // Update firstname/lastname for users
        update_user_meta( $user_id, 'first_name', $nameParts[0]);
        update_user_meta( $user_id, 'last_name', implode(' ', array_slice($nameParts, 1)));
    
        return $user_id;
    }

    function addUserRoleCaps($user_id, $role){
        global $wpdb;
 
        $user = new \WP_User( $user_id );

        $user->set_role($role);

        // // Add cache cap
        // $user->add_cap( 'rocket_purge_cache' );
        
        // // Add theme option cap
        // $user->add_cap( 'edit_theme_options' );

        // // Add translation cap
        // $user->add_cap( 'translate' );
        // $language_pairs = [
        //     'FR' => 'EN',
        //     'EN' => 'FR'
        // ];
        // update_user_meta( $user_id, $wpdb->prefix . 'language_pairs', $language_pairs );

    }

    function createBlog($title, $admin_user_id, $domain){
        
        if($admin_user_id === false) return false;
        $domain = str_replace(['http://', 'https://'],'', $domain);
        \ID3\write_log('create blog');

        if(class_exists('SitePress')){

            \ID3\Helpers\remove_class_filter('switch_blog','SitePress','init_settings');  
            \ID3\Helpers\remove_class_filter('pre_option_page_on_front','SitePress','pre_option_page_on_front');  
            //\ID3\Helpers\remove_class_filter('wpml_tm_save_post','Hooks','addToSavePostQueue');  
        }
        
        $blog_id = wp_insert_site([
            'domain' => $domain, 
            'title' => $title, 
            'user_id' => $admin_user_id
        ]);

        \ID3\write_log('after wp_insert_site');

        if(!$this->validateTarget($blog_id, $result)){
            //new \WP_Error(602, $result['error']);
            $this->throwError($result['error']);
            return $result;
        }

        if( is_wp_error($blog_id) ) {
            return $blog_id;
        }
        

        return $blog_id;
    }

    function syncContent($request){
        $data   = json_decode(json_encode($request->get_param('data')));
        $target = $data->site_id;
        $source = $data->source_site;
        //$synchables = $data->plugins->sync;
        \ID3\write_log('sync content');
        
        
        require_once \ID3\Aliquando\PLUGIN_DIR . '/core/class.sync.php';
        $result = [
            //'dir' => null,
            'db' => null
        ];

        // validate target before doing any thing harmfull
        if(!$this->validateTarget($target, $result)) {
            //new \WP_Error(602, $result['error']);
            $this->throwError($result['error']);
            return $result;
        }

        $siteSync = new SiteSync($source);

        //$result['dir'] = $siteSync->copy_source_files($target, $synchables);
        $result['db'] = $siteSync->copy_source_database($target);

        return $result;
    }

    function syncFiles($request){
        $data   = json_decode(json_encode($request->get_param('data')));
        $target = $data->site_id;
        $source = $data->source_site;
        //$synchables = $data->plugins->sync;

        \ID3\write_log('sync files');
        require_once \ID3\Aliquando\PLUGIN_DIR . '/core/class.sync.php';
        $result = [
            'dir' => null,
            //'db' => null
        ];

        // validate target before doing any thing harmfull
        if(!$this->validateTarget($target, $result)){
            //new \WP_Error(602, $result['error']);
            $this->throwError($result['error']);
            return $result;
        }

        $siteSync = new SiteSync($source);

        $result['dir'] = $siteSync->copy_source_files($target);
        //$result['db'] = $siteSync->copy_source_database($target, $synchables);

        return $result;
    }

    function addCustomLogo($title,$filename, $logo){

        add_filter( 'upload_mimes', function($mimes){
            $mimes['png'] = 'image/png';

            return $mimes;
        });

        $upload = \wp_upload_bits($filename, null, $logo);

        if (isset($upload['error']) && $upload['error']) {
            \ID3\write_log($filename . ' wp_upload_bits error return: ' . $upload['error']);
            return false;
        }
        $uploadFilePath = $upload['file'];
        $uploadFileType = '';
        if (!empty($upload['type'])) {
            $uploadFileType = $upload['type'];
        } else {
            $mime = \wp_check_filetype($uploadFilePath);
            if ($mime) {
                $uploadFileType = $mime['type'];
            }
        }
        $attachment = [
            'post_title' => $title, 
            'post_content' => $upload['url'], 
            'post_type' => 'attachment', 
            'post_mime_type' => $uploadFileType, 
            'post_status'    => 'inherit',
            'guid' => $upload['url']
        ];
        
        $attachment_id = \wp_insert_attachment($attachment, $uploadFilePath);
        if (!is_wp_error($attachment_id)) {
            if (!function_exists('wp_generate_attachment_metadata')) {
                // Make sure that this file is included, as wp_generate_attachment_metadata() depends on it.
                require_once ABSPATH . 'wp-admin/includes/image.php';
            }
            $attMeta = \wp_generate_attachment_metadata($attachment_id, $uploadFilePath);
            \wp_update_attachment_metadata($attachment_id, $attMeta);
        }

        \set_theme_mod('custom_logo', $attachment_id);
        return $attachment_id;
    }


    function validateTarget($target_id, &$message){
        if($target_id == null){
            $message['error'] = 'Target site is not defined';
            return false;
        }

        if($target_id == 1){
            $message['error'] = 'Target site is the primary site';
            return false;
        }

        if($target_id < \ID3\Aliquando\MIN_SITE_ID){
            $message['error'] = 'Target is under the minimum reserved site id. Received:' . $target_id . ' - Minimum:' . \ID3\Aliquando\MIN_SITE_ID;
            return false;
        }

        return true;
    }

    function throwError($message){
        header($_SERVER["SERVER_PROTOCOL"]." 602 " . $message);
        \ID3\write_log('ID3-Aliquando Error: ' . $message);
        die;
    }
}