<?php

namespace ID3\Aliquando;
/**
 * Plugin shortcodes handler
 * @package ID3\Aliquando 
 * For more infos, see: "Adding shortcode"
 */
class Shortcode extends \ID3\Bases\Shortcode{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        // define the list of shortcodes
        $list = [
            'social_medias' => 'socialMedia',   // add a value to key to target a different method
            'office_infos' => 'officeInfos',
            'office_address' => 'officeAddress',
            'office_phone' => 'officePhone',
            'office_email' => 'officeEmail',
            'network_content' => 'networkContent',
            'network_elementor_template' => 'networkElementorTemplate'
        ];

        $this->registerAll($list);
    }
    
    /**
     * Display social media list of link
     * Usage : [id3_aliquando_social_medias]
     *
     * @param string $class Optional
     * @param string $display icon|list
     */
    function socialMedia($atts, $content=null){
        extract(
            shortcode_atts([
                'class' => '',
                'display' => 'icon',
                'layout' => 'row'
            ], $atts)
        );

        $displayClasses = preg_split('/\W/i',$display);

        $classes = ['id3-aliquando-social-medias', $class, 'id3-layout-' . $layout, 'id3-aliquando-display-' . implode('-',$displayClasses) ];

        $settings = $this->getSettings();

        ob_start();

        if(count($settings->social_medias) > 0){
            
            echo('<div class="'. implode(' ', $classes) . '">');
            
            $typeLinks = [
                'facebook' => 'https://facebook.com/',
                'linkedin' => 'https://linkedin.com/',
                'twitter' => 'https://twitter.com/',
                'pinterest' => 'https://pinterest.com/'
            ];

            $typeIcons = [
                'facebook' => 'facebook-f'
            ];
            
            foreach($settings->social_medias as $item){
                $link = $typeLinks[$item->type] . $item->url;
                $iconClass = (isset($typeIcons[$item->type])) ? $typeIcons[$item->type] : $item->type;

                echo('<a href="' . $link . '" class="id3-aliquando-social-media-item" target="_blank">');
                $icon = '<i class="fab fa-fw fa-' . $iconClass . '"></i>';
                $enum = '<span>' . $item->type . '</span>';
                $url = '<span>' . $item->url . '</span>';

                echo(str_replace(
                    ['icon','enum','url'],
                    [$icon,$enum,$url],
                    $display));
                
                echo('</a>');
            }

            echo('</div>');
        }
        $result = ob_get_contents();
        ob_end_clean();
        return $result;
    }

   /**
     * Display office's informations
     * Usage : [id3_aliquando_office_infos]
     *
     * @param string $class Optional
     * @param string $icon Optional. 1 to display, 0 to hide. Default is 1
     */
    function officeInfos($atts, $content=null){
        extract(
            shortcode_atts([
                'class' => '',
                'icon' => '1'
            ], $atts)
        );

        $classes = implode(' ', ['id3-aliquando-office-infos', $class]);
    
        ob_start();


        echo "<div class='{$classes}'>";
        echo do_shortcode('[id3_aliquando_office_address icon="' . $icon . '"]');
        echo do_shortcode('[id3_aliquando_office_phone icon="' . $icon . '"]');
        echo do_shortcode('[id3_aliquando_office_email icon="' . $icon . '"]');
        echo "</div>";

        $result = ob_get_contents();
        ob_end_clean();

        return $result;
    }

    /**
     * Display office's address
     * Usage : [id3_aliquando_office_address]
     *
     * @param string $class Optional
     * @param string $icon Optional. 1 or icon name to display, 0 to hide. Default is 1
     */
    function officeAddress($atts, $content=null){
        extract(
            shortcode_atts([
                'class' => '',
                'icon' => '1'
            ], $atts)
        );

        $classes = implode(' ', ['id3-aliquando-office-address', $class]);
        $settings = $this->getSettings();
        
        // Bail if address is empty
        if($settings->office_address == '') return '';

        $encodeAddress = urlencode($settings->office_address);
        $iconTag = '';
        if($icon == '1'){
            $iconTag = '<i class="fal fa-fw fa-map-marker-alt"></i> ';
        }
        if(strpos($icon,'fa')===0){
            $iconTag = '<i class="fal fa-fw ' . $icon . '"></i> ';
        }
        
        return "<div class='{$classes}'>{$iconTag}<a href='maps:{$encodeAddress}' target='_blank'>{$settings->office_address}</a></div>";
    }

    /**
     * Display office's phone
     * Usage : [id3_aliquando_office_phone]
     *
     * @param string $class Optional
     * @param string $icon Optional. 1 or icon name to display, 0 to hide. Default is 1
     */
    function officePhone($atts, $content=null){
        extract(
            shortcode_atts([
                'class' => '',
                'icon' => '1'
            ], $atts)
        );

        $classes = implode(' ', ['id3-aliquando-office-phone', $class]);
        $settings = $this->getSettings();
        // Bail if phone is empty
        if($settings->office_phone == '') return '';

        $iconTag = '';
        if($icon == '1'){
            $iconTag = '<i class="fal fa-fw fa-phone"></i> ';
        }
        if(strpos($icon,'fa')===0){
            $iconTag = '<i class="fal fa-fw ' . $icon . '"></i> ';
        }

        return "<div class='{$classes}'>{$iconTag}<a href='tel:{$settings->office_phone}'>{$settings->office_phone}</a></div>";
    }

    /**
     * Display office's email
     * Usage : [id3_aliquando_office_email]
     *
     * @param string $class Optional
     * @param string $icon Optional. 1 or icon name to display, 0 to hide. Default is 1
     */
    function officeEmail($atts, $content=null){
        extract(
            shortcode_atts([
                'class' => '',
                'icon' => '1'
            ], $atts)
        );

        $classes = implode(' ', ['id3-aliquando-office-email', $class]);
        $settings = $this->getSettings();
        
        // Bail if email is empty
        if($settings->office_email == '') return '';

        $iconTag = '';
        if($icon == '1'){
            $iconTag = '<i class="fal fa-fw fa-envelope"></i> ';
        }
        if(strpos($icon,'fa')===0){
            $iconTag = '<i class="fal fa-fw ' . $icon . '"></i> ';
        }

        return "<div class='{$classes}'>{$iconTag}<a href='mailto:{$settings->office_email}'>{$settings->office_email}</a></div>";
    }

    /**
     * Render content from a network post_id (page,post)
     * Usage : [id3_aliquando_network_content id="11"]
     *
     * @param string $class Optional
     */
    function networkContent($atts, $content=null){
        extract(
            shortcode_atts([
                'id' => '',
                'site_id' => 1
            ], $atts)
        );
        if($id == '') return 'Please select a content';
        
        //$settings = $this->getSettings();   
        switch_to_blog($site_id);
        $content = get_the_content(null,false, $id);
        $result = do_shortcode( $content );
        restore_current_blog();

        return $result;
    }

    /**
     * Render content from a network post_id (page,post,template,section,etc)
     * Usage : [id3_aliquando_network_elementor_template id="12"]
     *
     * @param string $class Optional
     */
    function networkElementorTemplate($atts, $content=null){
        extract(
            shortcode_atts([
                'id' => '',
                'site_id' => 1
            ], $atts)
        );

        if($id == '') return 'Please select a content';

        $result = '';
        $settings = $this->getSettings();

        switch_to_blog($site_id);

		if ( 'publish' == get_post_status( $id ) ) {
            // Remove all filter to url in content
            if(!is_admin() && !is_user_logged_in()){
                if(class_exists('WPML_Elementor_URLs')){
                    $removedFilters = \ID3\Helpers\remove_class_filter('elementor/document/urls/edit','WPML_Elementor_URLs','adjust_edit_with_elementor_url');  
                }
            }

            $localizedId = apply_filters( 'wpml_object_id', $id, 'elementor_library' );
            $result = '<div class="elementor-template">';
            $result .= \ElementorPro\Plugin::elementor()->frontend->get_builder_content_for_display( $localizedId , true);
            $result .= '</div>';
            
        }

        restore_current_blog();        

        return $result;
    }
}