<?php
namespace ID3\Aliquando;

class SiteSync{

    //public $target_site_id = null;
	public $source_site_id = null;
	public $source_site = null;
	private static $_blog_exists_cache = [];
	
    public function __construct($from){
        $this->source_site_id = $from;
		//$this->target_site_id = $to;
		
		$this->load_source();
	}
	
	public function load_source(){
		$this->source_site = get_site($this->source_site_id);

		switch_to_blog($this->source_site_id);

		$this->source_site->upload_dir = wp_upload_dir()['basedir'];
		$this->source_site->settings = \ID3\Settings::get(KEY);

		restore_current_blog();
	}

    public function copy_source_database($target_site_id){
        global $wpdb;
		
		if($target_site_id == null) return false;
		if($target_site_id == 1) return false;
		if($target_site_id < 5) return false;

		\ID3\write_log('copy_source_database:switch to ' . $target_site_id);
		switch_to_blog($target_site_id);

		//$target_site_id = get_current_blog_id();

		// get source and destination tables name (without "ali_[site_id]_")
		$source_tables = $this->get_site_table_list($this->source_site_id);
		$dest_tables = $this->get_site_table_list();


		$source_prefix = $wpdb->get_blog_prefix($this->source_site_id);
		$target_prefix = $wpdb->get_blog_prefix($target_site_id);

		$option_name_exceptions = [
			'blogname','blogdescription', 'admin_email',
			'siteurl','home', 'upload_path',
			'template','stylesheet' 
			//,$source_prefix . 'user_roles', $target_prefix . 'user_roles'
		];
		
		$tables_affected = array();
		$tables_affected['sources'] = $this->get_site_table_list($this->source_site_id, true);
		
        // get source/destination siteUrl for further replace
        $source_site_url = $this->get_option('siteurl',$this->source_site_id);
        $target_site_url = $this->get_option('siteurl');

		if(strpos($target_site_url,'//')===false){
			$target_site_url = '//' . $target_site_url;
		}
		
		if($source_prefix == $target_prefix){
			
			return new WP_Error(602, 'Source and target are the same');
		}

		foreach ($source_tables as $table) {
			$truncOrCreate = 'skipped';		

			if($source_prefix . $table == $target_prefix . $table){
				continue;
			}

			if($table == ''){
				$tables_affected[] = array('name'=>$table, 'rows'=>'Table name empty', 'truncOrCreate'=>'Skipped');
				continue;
			}

			if(in_array($table, $dest_tables)){
				// if table exists, truncate it
				if($table != 'options'){
					$wpdb->query("TRUNCATE TABLE {$target_prefix}{$table}");
				}
				else{
					$wpdb->query("DELETE FROM {$target_prefix}{$table} WHERE option_name NOT IN ('" . implode("','", $option_name_exceptions)  . "')");
				}
				$truncOrCreate = 'truncated';
			}
			else{
				// if it doesn't, create it
				$creation_query = $this->get_table_creation_sql($table, $this->source_site_id, $target_site_id);
				if($creation_query != null){
					$wpdb->query($creation_query);
					$truncOrCreate = 'created';
				}
				else{
					$tables_affected[] = array('name'=>$table, 'rows'=>$this->get_table_creation_sql($table, $this->source_site_id, $target_site_id,true), 'truncOrCreate'=>'Unable to create');
				}
			}

			if($truncOrCreate != 'skipped'){
				$affected = true;
				
				switch ($table) {
					case 'options':
						$lSql = "UPDATE {$target_prefix}{$table} SET option_value=REPLACE(option_value, '{$source_site_url}','{$target_site_url}') WHERE option_name IN ('siteurl','home','fileupload_url','dashboard_widget_options')";
						$wpdb->query($lSql);
						
						$lSql = "INSERT INTO {$target_prefix}{$table} SELECT * FROM {$source_prefix}{$table} WHERE option_name NOT IN ('" . implode("','", $option_name_exceptions)  . "')";
						$affected = $wpdb->query($lSql);

						if($affected===false){
							$affected = $wpdb->last_error . ' on ' . $lSql ;
						}
						else{
							$lSql = "UPDATE {$target_prefix}{$table} set option_name='{$target_prefix}user_roles' where option_name='{$source_prefix}user_roles' ";
							$affected = $wpdb->query($lSql);
						}
						break;
					default:
					 
						$lSql = "INSERT INTO {$target_prefix}{$table} SELECT * FROM {$source_prefix}{$table}";
						$affected = $wpdb->query($lSql);

						if($affected===false){
							$affected = $wpdb->last_error . ' on ' . $lSql ;
						}
				}

				$tables_affected[] = array('name'=>$table, 'rows'=>$affected, 'truncOrCreate'=>$truncOrCreate);

			}
		}

		restore_current_blog();

		return $tables_affected;
    }

    public function copy_source_files($target_site_id){
        $result = [];
        $target_dir_infos = '';
        $source_dir_infos = '';

		// get copy recursivity based on source site media sharing status
		// Bail now if source site is sharing it's media
		if(isset($this->source_site->settings) 
			&& isset($this->source_site->settings->share_media) 
			&& $this->source_site->settings->share_media === true){
			return $result;
		}

		switch_to_blog($target_site_id);
		
        try {
			$source_dir_infos = $this->source_site->upload_dir;
            $target_dir_infos = $this->get_upload_dir();
			
			$result['from'] = $source_dir_infos;
			$result['to'] 	= $target_dir_infos;


			if($source_dir_infos != null && $target_dir_infos != null){
				$this->copy_directory($source_dir_infos, $target_dir_infos);
			}
            
		}
		catch(Exception $e){
            $result = new WP_Error(601,'Unable to copy source files(' . $source_dir_infos . ') to destination(' . $target_dir_infos . ')', $e);
		}
		
		restore_current_blog();
        return $result;
    }

    public static function update_content($source_site_id, $content_id, $content_type='page'){
        
    }

    /** TOOLS */

	// ---------
	// Database
	// ---------

	function blog_exists( $site_id = null ) {
        global $wpdb;
        $site_id = (int) $site_id;
		$result = true;

        if ( null === $site_id )
			return true;
			
		if ( wp_is_large_network() ) // we do not test large sites.
			return true;

		if(in_array($site_id, self::$_blog_exists_cache)){
			return self::$_blog_exists_cache[$site_id];
		}

		$query = "SELECT `blog_id` FROM $wpdb->blogs
				WHERE site_id = {$site_id} AND deleted = 0";

		$query_result = $wpdb->get_col( $query );
		
		if ( empty ( $query_result ) ) $result = false;
		
		self::$_blog_exists_cache[$site_id] = $result;
		return $result;
    }

	function get_table_creation_sql($tablename, $source_id, $dest_id, $debug=false){
		global $wpdb;
		$prefix = $wpdb->get_blog_prefix($source_id);
		$target_prefix = $wpdb->get_blog_prefix($dest_id);

		
		$schema = $wpdb->get_results("SHOW CREATE TABLE " . DB_NAME . ".{$prefix}{$tablename}");
		$result = null;
		if($schema!=null){
			$result = str_replace($prefix ,$target_prefix, $schema[0]->{'Create Table'});
		}

		if($debug==true){
			return [
				'sql' => "SHOW CREATE TABLE  " . DB_NAME . ".{$prefix}{$tablename}",
				'result' => $result
			];
		}

		return $result;
	}

	function get_site_table_list($site_id=null, $debug=false){
		global $wpdb;
		
		$prefix = $wpdb->get_blog_prefix($site_id);

		$sql = "SELECT TABLE_NAME FROM information_schema.tables WHERE table_schema = '" . DB_NAME ."' and table_name  REGEXP '^{$prefix}[[:alpha:]]'";
		$tables = $wpdb->get_results($sql);

		
		
		$result = array();
		foreach ($tables as $rows) {
			$tableName = str_replace($prefix, '', $rows->TABLE_NAME);

			// skip those tables since they are refering to global network data
			if(in_array($tableName, ['blogs','blogmeta','site','sitemeta'])){
				continue;
			}

			$result[] = $tableName;
		}

		if($debug==true){
			return [
				'sql' => $sql,
				'sql_result' => $tables,
				'result' => $result
			];
		}
		return $result;
	}

	public function get_option($option_name,$site_id=null){
		$switch = false;
		
		if($site_id!=null && !$this->blog_exists($site_id)) return null;
		
        if ($site_id!=null) {
            $switch = true;
            switch_to_blog( $site_id );
		}
		
		$result = get_option($option_name);

		if ( $switch ) {
            restore_current_blog();
		}
		
		return $result;
	}

    // -------
    //   I/O
    // -------

    // copy a folder from one location to another, recursively*
    public function copy_directory($from, $to, $recursive=true){
        if ( is_dir( $from ) ) {
			if(!file_exists($to)){
				mkdir( $to );
			}
	        
	        $d = dir( $from );
	        while ( FALSE !== ( $entry = $d->read() ) ) {
	            if (in_array($entry, ['.','..','sites'])) {
	                continue;
	            }
				$sourceEntry = $from . '/' . $entry; 
				$targetEntry = $to . '/' . $entry;

	            if ( is_dir( $sourceEntry ) && $recursive) {
	                $this->copy_directory( $sourceEntry, $targetEntry,$recursive);
	                continue;
	            }

	            if(!file_exists($targetEntry)){
		            copy( $sourceEntry, $targetEntry );
		        }
	        }

	        $d->close();
	    }else {
	    	if(!file_exists($to)){
		        copy( $from, $to );
		    }
	    }
    }

    // Get the upload directory of a specific site
    public function get_upload_dir($site_id=null){
        $switch = false;
		
		if($site_id!=null && !$this->blog_exists($site_id)) return null;
		
        if ( $site_id != null && get_current_blog_id() != $site_id ) {
            $switch = true;
            switch_to_blog( $site_id );
        }
	   
		$dirInfos = wp_upload_dir();
		
        $blog_dir_infos = $dirInfos['basedir'];
        
        if ( $switch ) {
            restore_current_blog();
        }
        return $blog_dir_infos;
    }

}