<?php
namespace ID3\Aliquando\Modules\Elementor;

class OfficeInfoWidget extends \Elementor\Widget_Base {

    private $_module = null;
    
    public function setModule($module){
        $this->_module = $module;
    }

	public function get_name() {
        return 'ali_office_info';
    }

	public function get_title() {
        return __('Office Info', \ID3\Aliquando\TDOM);
    }

	public function get_icon() {
        return 'fa fa-building';
    }

	public function get_categories() {
        return ['aliquando'];
    }

	protected function _register_controls() {

        
        $this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content'),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
        );

        
        $list = [
            'office_infos' => 'All',
            'office_address' => 'Address',
            'office_phone' => 'Phone',
            'office_email' => 'Email',
        ];
        
        
		$this->add_control(
			'shortcode',
			[
				'label' => __( 'Displayed information', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::SELECT,
                'placeholder' => '',
                'options' => $list,
                'default' => 'office_infos'
			]
        );

        $this->add_control(
			'show_icon',
			[
				'label' => __( 'Show icon', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::SWITCHER,
                'placeholder' => '',
                'return_value' => 'on',
                'default' => 'on'
			]
        );

        $this->add_control(
			'custom_icon',
			[
				'label' => __( 'Custom icon', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::TEXT,
                'placeholder' => 'fa-check',
                'default' => '',
                'conditions' => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name' => 'shortcode' ,
                            'operator'=> '!=',
                            'value' => 'office_infos'
                        ],
                        [
                            'name' => 'show_icon' ,
                            'operator'=> '==',
                            'value' => 'on'
                        ]
                    ]
                ]
			]
        );

        
		$this->end_controls_section();

        $this->start_controls_section(
			'style_section',
			[
				'label' => __( 'Style Section', 'plugin-name' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
			]
		);

        $this->start_controls_tabs(
			'style_tabs'
		);

		// $this->start_controls_tab(
		// 	'style_tab',
		// 	[
		// 		'label' => __( 'Styles', \ID3\Aliquando\TDOM ),
		// 	]
		// );

        $this->add_control(
			'element_color',
			[
				'label' => __( 'Color', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::COLOR,
				'scheme' => [
					'type' => \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
				'selectors' => [
					'{{WRAPPER}} .id3-aliquando-office-infos' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'content_typography',
				'label' => __( 'Typography', \ID3\Aliquando\TDOM ),
				'scheme' => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_3,
				'selector' => '{{WRAPPER}} .id3-aliquando-office-infos',
			]
		);

        //$this->end_controls_tab();
        $this->end_controls_tabs();

		$this->end_controls_section();

    }

    

    /**
	 * Render shortcode widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
        
        $settings = $this->get_settings();
        
        $shortcode = $settings['shortcode'];
        $show_icon = $settings['show_icon'];
        $custom_icon = $settings['custom_icon'];
        if ($shortcode == ''){
            echo('Select something to display');
            return;
        }
        $shortcode_attrs = [];
        if( $show_icon == 'on'){
            if($custom_icon != ''){
                $shortcode_attrs[] = 'icon="' .  $custom_icon . '"';
            }
        }
        else{
            $shortcode_attrs[] = 'icon="0"';
        }

        $shortcodeName = 'id3_aliquando_' . $shortcode;
        $result = do_shortcode( shortcode_unautop('[' . $shortcodeName . ' ' . implode(' ', $shortcode_attrs) . ']' ));
		
        echo( $result );
    }

	

}