<?php
namespace ID3\Aliquando\Modules\Elementor;

class SocialMediasWidget extends \Elementor\Widget_Base {

    private $_module = null;
    
    public function setModule($module){
        $this->_module = $module;
    }

	public function get_name() {
        return 'ali_social_medias';
    }

	public function get_title() {
        return __('Social medias', \ID3\Aliquando\TDOM);
    }

	public function get_icon() {
        return 'eicon-social-icons';
    }

	public function get_categories() {
        return ['aliquando'];
    }

	protected function _register_controls() {

        
        $this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content'),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
        );

        
        $layoutList = [
            'row' => 'Row',
            'column' => 'Column'
        ];

        $this->add_control(
			'layout',
			[
				'label' => __( 'Orientation', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::SELECT,
                'placeholder' => '',
                'options' => $layoutList,
                'default' => 'row'
			]
        );

        $this->add_control(
			'spacing',
			[
				'label' => __( 'Spacing', \ID3\Aliquando\TDOM ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px', 'em', 'rem' ],
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 200,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 10,
				],
                'selectors' => [
					'{{WRAPPER}} .id3-aliquando-social-medias' => '--list-gap: {{SIZE}}{{UNIT}}',
				],
			]
		);
        
        $this->add_control(
			'format',
			[
				'label' => __( 'Element format', \ID3\Aliquando\TDOM),
                'label_block' => true,
                'description' => __('Ex.: icon enum:url', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::TEXT,
                'placeholder' => '',
                'default' => 'icon'
			]
        );
        
		$this->end_controls_section();

        $this->start_controls_section(
			'style_section',
			[
				'label' => __( 'Style Section', 'plugin-name' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
			]
		);

        $this->start_controls_tabs(
			'style_tabs'
		);

		// $this->start_controls_tab(
		// 	'style_tab',
		// 	[
		// 		'label' => __( 'Styles', \ID3\Aliquando\TDOM ),
		// 	]
		// );

        $this->add_control(
			'element_color',
			[
				'label' => __( 'Color', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::COLOR,
				'scheme' => [
					'type' => \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
				'selectors' => [
					'{{WRAPPER}} .id3-aliquando-social-medias' => 'color: {{VALUE}}',
				],
			]
		);
        $this->add_control(
			'element_color_hover',
			[
				'label' => __( 'Hover color', \ID3\Aliquando\TDOM),
				'type' => \Elementor\Controls_Manager::COLOR,
				'scheme' => [
					'type' => \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_4,
				],
				'selectors' => [
					'{{WRAPPER}} .id3-aliquando-social-medias a:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' => 'content_typography',
				'label' => __( 'Typography', \ID3\Aliquando\TDOM ),
				'scheme' => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_3,
				'selector' => '{{WRAPPER}} .id3-aliquando-social-medias',
			]
		);


        //$this->end_controls_tab();
        $this->end_controls_tabs();

		$this->end_controls_section();

    }

    

    /**
	 * Render shortcode widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
        
        $settings = $this->get_settings();
        
        $layout = $settings['layout'];
        $format = $settings['format'];
        

        $shortcode_attrs = [];
        $shortcode_attrs[] = 'display="' . $format . '"';
        $shortcode_attrs[] = 'layout="' . $layout . '"';
        
        $shortcodeName = 'id3_aliquando_social_medias';
        $result = do_shortcode( shortcode_unautop('[' . $shortcodeName . ' ' . implode(' ', $shortcode_attrs) . ']' ));
		
        echo( $result );
    }

	

}