const $licenseTypes = [
    {key: 'google_map', label: 'Google Maps'},
    {key: 'local_logic', label: 'Local Logic'}
];
const $codeFragmentParts = [
    {key: 'head:start', label: 'Start of <head>'},
    {key: 'head:end', label: 'End of <head>'},
    {key: 'body:start', label: 'Start of <body>'},
    {key: 'body:end', label: 'End of <body>'},
    {key: 'wp_action', label: 'WordPress action hook'}
];

const $codeFragmentLanguages = [
    {key: 'html', label: 'Plain HTML'},
    {key: 'php', label: 'Server script (PHP)'},
    {key: 'js', label: 'Javascript'},
    {key: 'css', label: 'CSS'},
];

const $codeFragmentConditionTypes = [
    {key: 'locale', label: 'Locale'},
    {key: 'postId', label: 'Post/page ID'},
    {key: 'slug', label: 'Slug'},
    {key: 'path', label: 'Path'},
];

id3Plugin
.controller('aliquandoCtrl', 
function(
    $scope, $rootScope, $q, $timeout, 
    $id3UI, $id3Rest, $id3Settings, $id3Lists){

    $scope.isReady = false;
    $scope.init_completed = false;
    $scope.networkSettings = {
        available_sources: [],
        licenses: []
    }

    $scope.display = {
        requirements: []
    };

    $scope.settings = {
        source_site : '',
        theme_modifier: '',
        color_palettes: {},
        cookie_consent: $id3PluginContext.is_multisite ? 'inherit' : 'none',
        cookie_consent_delay: 0,
        privacy_policies_page: null,
        office_address: '',
        office_phone: '',
        office_email:'',
        share_media: false,
        social_medias: [],
        requirements: [],
        licences: [],
    };

    $scope.licenseDialogOptions = {}
    
    $scope.init = function(){
        
        $rootScope.isReady({moduleName: 'aliquando'}).then(_ => {
            $scope.isMultisite = $id3PluginContext.is_multisite;

            $id3Rest.get('aliquando/theme/list').then($response => {
                $id3Lists.register('themeModifiers', Object.keys($response).map($k => {
                    return {
                        key: $k,
                        label: $response[$k]
                    }
                }))
            });

            $id3Lists.register('socialMedias', [
                {key: 'facebook', label : 'Facebook', url: 'facebook.com'},
                {key: 'twitter', label : 'Twitter', url: 'twitter.com'},
                {key: 'linkedin', label : 'LinkedIn', url: 'linkedin.com'},
                {key: 'pinterest', label : 'Pinterest', url: 'pinterest.com'},
                {key: 'instagram', label : 'Instagram', url: 'instagram.com'},
                {key: 'youtube', label : 'YouTube', url: 'youtube.com'}
            ]);

            $id3Lists.register('licenseTypes', $licenseTypes);

            $id3Lists.register('codeFragmentParts', $codeFragmentParts);
            $id3Lists.register('codeFragmentLanguages', $codeFragmentLanguages);
            $id3Lists.register('codeFragmentConditionTypes', $codeFragmentConditionTypes);

            $q.all([
                $id3Settings.assignTo($scope,'networkSettings',true),
                $id3Settings.load($scope,'settings'),
                $scope.fetchThirdPartyConfigs()
            ])
            .then(_ => {
               
                console.log('Aliquando is ready to do stuff with', $scope.settings);

                $scope.fetchSourceSiteInfos();
                $scope.licenseDialogOptions.params = {
                    networkLicenses: $scope.networkSettings.licenses
                }
                $scope.init_completed = true;
                $scope.isReady = true;
            });
        });

    }

    $scope.fetchThirdPartyConfigs = function(){
        return $q.all([
            $id3Rest.get('aliquando/third-party/optionals')
            //$id3Rest.get('aliquando/third-party/requirements'),
        ])
            .then( ([$optionals,$requirements]) => {
                // $scope.display.requirements = $requirements.map($r => {
                //     return {path : $r.path}
                // });
                $scope.display.optionals = $optionals.map($r => {
                    return {path : $r.path}
                });

                // $scope.settings.requirements = $requirements.reduce( ($result, $cursor, $index) => {
                //     if($cursor.defaults == undefined) return $result;
                //     return Object.assign($cursor.defaults, $result);
                // },$scope.settings.requirements);

                $optionals.forEach($opt => {
                    if($opt.defaults == undefined) return;
                    Object.keys($opt.defaults).forEach($k => {
                        if($scope.settings[$k] != undefined) return;
                        $scope.settings[$k] = $opt.defaults[$k];
                    })
                })
            })
    }

    $scope.fetchSourceSiteInfos = function(){
        if($scope.settings.source_site == '') return ;

        $id3Rest.get('aliquando/sites/get', {id: $scope.settings.source_site}).then($siteInfos => {
            $scope.sourceSiteInfos = $siteInfos;
        });
    }

    

    $scope.searchSourceSite = function(){
        $id3UI.sidePanel('search-site',{source_only:true}).then($result => {
            console.log('searchSourceSite',$result, $scope.settings);
            $scope.settings.source_site = $result.id;
            $scope.fetchSourceSiteInfos();
        });
    }

    $scope.editPalette = function(){
        let lPalette = angular.copy($scope.settings.color_palettes);
        if(lPalette == undefined || Array.isArray(lPalette)){
          lPalette = {};  
        }
        $id3UI.dialog('color-palette',lPalette).then($new => {
            $scope.settings.color_palettes = $new;
            
        })
    }

    $scope.openSourceSite = function($site_id){
        window.open('//' + $id3PluginContext.networkDomainName + '/wp-admin/network/site-info.php?id=' + $site_id);
    }

    $scope.isSourceSite = function(){
        if($id3PluginContext.currentBlogId == 1) return true;
        if($scope.networkSettings == undefined || $scope.networkSettings == false) return false;
        
        
        return $scope.networkSettings.available_sources.some($source => $source.id == $id3PluginContext.currentBlogId);
    }

    $scope.setup = function(){
        $id3UI.wizard('setup').then( $setupInfo => {
            $id3UI.progressPanel([
                {label:'Saving plugin data'     , action: () => {
                    $scope.settings.social_medias   = $setupInfo.social_medias;
                    $scope.settings.office_address  = $setupInfo.office_address;
                    $scope.settings.office_phone    = $setupInfo.office_phone;
                    $scope.settings.office_email    = $setupInfo.office_email;
                }},
                {label:'Creating pages'         , action: ($previousStepResult) => $id3Rest.post('aliquando/pages/apply-structure', {data: Object.assign($setupInfo, $previousStepResult)})},
            ],{mode: 'chain', onFail: $error => {
                $id3UI.alert('Site setup process failed : ' + $error.statusText);
            }})
            .then( 
                $finalResult => {
                    console.log('setup completed', $result);
                    console.table($finalResult);
                    
                    $id3UI.alert('Site setup complete');

                },
                ($failError) => {
                    $id3UI.alert('Site setup process failed : ' + $failError.statusText);
                    console.log($failError);
                }
            )
            .catch( ($error) => {
                $id3UI.alert('Site setup process failed : ' + $error.statusText);
                //$id3UI.toast('Site creation failed',$error);
            });
        })
    }
});


id3Plugin
.controller('aliquandoNetworkCtrl', 
function(
    $scope, $rootScope, $q, $timeout, 
    $id3UI, $id3Rest, $id3Settings, $id3Lists){

    $scope.settings = {
        block: {
            wpSettings: false,
            wpPlugins: false,
            wpThemes: false,
        },
        cookie_consent: 'none',
        cookie_consent_delay: 0,
        registrationLevel: 'admin',
        available_sources: [],
        licenses: []
    };

    $scope.newSiteTemplate = {
        title: '',
        source : null
    }
    
    $scope.siteCreationIcon = {
        admin: 'users-crown',
        protected: 'user-lock',
        public: 'user-friends'
    }

    $scope.init = function(){
        $scope.newSite = angular.copy($scope.newSiteTemplate);
        
        $id3Lists.register('licenseTypes', $licenseTypes);
        $id3Lists.register('socialMedias', [
            {key: 'facebook', label : 'Facebook', url: 'facebook.com'},
            {key: 'twitter', label : 'Twitter', url: 'twitter.com'},
            {key: 'linkedin', label : 'LinkedIn', url: 'linkedin.com'},
            {key: 'pinterest', label : 'Pinterest', url: 'pinterest.com'},
            {key: 'instagram', label : 'Instagram', url: 'instagram.com'},
            {key: 'youtube', label : 'YouTube', url: 'youtube.com'}
        ]);

        $id3Lists.register('codeFragmentParts', $codeFragmentParts);
        $id3Lists.register('codeFragmentLanguages', $codeFragmentLanguages);
        $id3Lists.register('codeFragmentConditionTypes', $codeFragmentConditionTypes);
        
        $rootScope.isReady({moduleName: 'aliquando'}).then(_ => {
            $scope.isMultisite = $id3PluginContext.is_multisite;

            $id3Rest.get('aliquando/theme/list').then($response => {
                $id3Lists.register('themeModifiers', Object.keys($response).map($k => {
                    return {
                        key: $k,
                        label: $response[$k]
                    }
                }))
            });

            $id3Settings.load($scope,'settings').then(_ => {
                
            });

            // $id3Settings.load($scope,'settings').then(_ => {
                
            // });
        });

    }

    $scope.addLicense = function(){
        const lNewItem = {
            key: $id3Lists.get('licenseTypes').first().key,
            value: '',
            activation_required: true,
            overridable: true
        };

        $scope.editLicense(lNewItem).then($item => {
            $scope.settings.licenses.push($item);
        })
    }

    $scope.editLicense = function($item){
        return $id3UI.dialog('network-license', $item);
    }
    $scope.removeLicense = function($item){
        $id3UI.confirm('Are you sure you want to remove this license?').then(_ => {
            $scope.settings.licenses = $scope.settings.licenses.filter($l => $l != $item);
        })
    }

    $scope.addSourceSite = function(){
        $id3UI.sidePanel('search-site',{source_only:false}).then($result => {
            console.log('searchSourceSite',$result, $scope.settings);
            if($scope.settings.available_sources == undefined)$scope.settings.available_sources = [];
            if($scope.settings.available_sources.find($s => $s.id == $result.id) == null){
                $scope.settings.available_sources.push($result);
            }
        });
    }
    
    $scope.removeSourceSite = function($site){
        $id3UI.confirm('Are you sure you want to remove this site as source?').then(_ => {
            $scope.settings.available_sources = $scope.settings.available_sources.filter($s => $s.id != $site.id);

        })
    }


    $scope.addNewSite = function(){
        
        $id3UI.wizard('new-site').then($result => {

            $id3UI.progressPanel([
                {label:'Site creation'          , action: () => $id3Rest.post('aliquando/site/create', {data: $result})},
                {label:'Synchronizing content'  , action: ($previousStepResult) => $id3Rest.post('aliquando/site/create/sync-content', {data: Object.assign($result, $previousStepResult)})},
                {label:'Synchronizing files'    , action: ($previousStepResult) => $id3Rest.post('aliquando/site/create/sync-files', {data: Object.assign($result, $previousStepResult)})},
                {label:'Applying theme'         , action: ($previousStepResult) => $id3Rest.post('aliquando/site/create/apply-theme', {data: Object.assign($result, $previousStepResult)})},
                {label:'Finalizing'             , action: ($previousStepResult) => $id3Rest.post('aliquando/site/create/finalize', {data: Object.assign($result, $previousStepResult)})}
            ],{mode: 'chain', onFail: $error => {
                $id3UI.alert('Site creation process failed : ' + $error.statusText);
            }})
            .then( 
                $finalResult => {
                    console.log('creation completed', $result);
                    console.table($finalResult);
                    
                    $id3UI.dialog('site-post-creation', $result).then($redirection => {
                        if($redirection == null) return;
                        window.open($redirection, 'new-site');
                    });
                },
                ($failError) => {
                    $id3UI.alert('Site creation process failed : ' + $failError.statusText);
                    console.log($failError);
                    // rollback site creation
                    if($failError.site_id){
                        $id3Rest.post('aliquando/site/delete', {site_id: $create.site_id});
                    }
                }
            )
            .catch( ($error) => {
                $id3UI.alert('Site creation process failed : ' + $error.statusText);
                //$id3UI.toast('Site creation failed',$error);
            });
        
            
        })
    }

});

