id3Plugin
.controller('newSiteWizardCtrl', 
function($scope, $rootScope, $q, $timeout, $id3Settings,$id3Lists,$id3Rest){
    $scope.settings = {
        available_sources: [],
    };
    $scope.theme_settings = {
        requirements: []
    }
    $scope.networkDomainName = '';

    $scope.model = {
        title : '',
        addDescription: false,
        description: '',
        source_site: null,
        theme:null,
        create_user: false,
        user: {
            name: null,
            email: null,
            password: null
        },
        requirements:[],
        social_medias: []
    }

    $scope.socialMedias =  $id3Lists.get('socialMedias');

    $scope.flags = {
        isValidTitle : 'times-circle',
        isValidDomain : 'times-circle',
    }

    $scope.init = function(){
        $id3Settings.load($scope,'settings').then(_ => {
               console.log('newSiteWizardCtrl/init', $scope.settings);
        });
        $scope.networkDomainName = $id3PluginContext.networkDomainName;

        $id3Rest.get('themes').then( $response => {
            $scope.themes = $response.filter($t => {
                if($response.some($ct => $ct.parent_theme == $t.name)){
                    return false;
                }
                return true;
            });
        });

        $scope.registerPages([
            { key: 'basic',             path: 'new-site/basic' },
            { key: 'domain',            path: 'new-site/domain' },
            { key: 'theme',             path: 'new-site/theme' },
            { key: 'requirements',      path: 'new-site/requirements' },
            { key: 'syncing',           path: 'new-site/syncing' },
            { key: 'users',             path: 'new-site/users' },
            { key: 'social-medias',     path: 'new-site/social-medias' },
            
        ]);

        $scope.registerValidation('basic', _ => $scope.validatePage('basic'));
        $scope.registerValidation('syncing', _ => $scope.validatePage('syncing'));
        $scope.registerValidation('domain', _ => $scope.validatePage('domain'));
        $scope.registerValidation('theme', _ => $scope.validatePage('theme'));
        $scope.registerValidation('users', _ => $scope.validatePage('users'));
        $scope.registerValidation('requirements', _ => $scope.validatePage('requirements'));

        $scope.registerAction('buildResult', $scope.buildResult);

        $scope.resetSocialModel();
    }

    $scope.buildResult = function(){
        console.log('newSiteWizard/buildResult',$scope.model);

        return $q( ($resolve,$reject) => {
            $id3Rest.get('settings/get',{plugin: $id3PluginContext.key, site_id: $scope.model.source_site}).then($sourceSetting => {
                const lDomain = $scope.model.isFullDomain===true
                                ? $scope.model.domain
                                : $scope.model.domain + '.' + $id3PluginContext.networkDomainName;
                                
                $scope.model.domain = lDomain;
                $scope.model.plugins = [
                    {
                        name: $id3PluginContext.key, 
                        settings: Object.assign($sourceSetting, {
                            social_medias: $scope.model.social_medias,
                            source_site: $scope.model.source_site,
                            office_address : $scope.model.office_address,
                            office_phone : $scope.model.office_phone,
                            office_email : $scope.model.office_email,
                            share_media: undefined,
                            sync: ['pages','menus','content','layout']
                        })
                    }
                ];

                delete $scope.model.social_medias;
                delete $scope.model.office_address;
                delete $scope.model.office_phone;
                delete $scope.model.office_email;

                return $resolve($scope.model,null);
            })
        });
        
    }

    $scope.validatePage = function($key){
        const lHandler = {
            'basic': _ => {
                return $scope.flags.isValidTitle == 'check-circle';
            },
            'domain': _ => {
                return $scope.flags.isValidDomain == 'check-circle';
            },
            'syncing': _ => {
                return $scope.model.source_site!=null;
            },
            'users': _ => {
                if(!$scope.model.create_user) return true;

                return !id3IsNullOrEmpty($scope.model.user.name)
                        && !id3IsNullOrEmpty($scope.model.user.email)
                        && !id3IsNullOrEmpty($scope.model.user.password);
            },
            'theme': _ => {
                return $scope.model.theme!=null;
            },
            'requirements' : _ => {
                const lThirdPartyInputs = Array.from(document.querySelectorAll('.id3-third-party-block .id3-input.id3-required'));
                
                const lResult = !id3IsNullOrEmpty($scope.model.office_address)
                        && !id3IsNullOrEmpty($scope.model.office_email)
                        && !id3IsNullOrEmpty($scope.model.office_phone)
                        && lThirdPartyInputs.every($input => !id3IsNullOrEmpty($input.value));

                console.log('requirements page validation', lResult);
                return lResult;
            }
        };

        console.log('validation for page', $key);
        if(lHandler[$key] == undefined) return true;
        return lHandler[$key]();
    }

    // Basic
    $scope.validateSiteTitle = function(){
        if(id3IsNullOrEmpty($scope.model.title)){
            $scope.flags.isValidTitle = 'times-circle';
            $scope.flags.isValidTitleMsg = 'Empty title';
            return;
        } 

        $scope.flags.isValidTitle = 'spinner-third fa-spin';
        $id3Rest.post('aliquando/sites/search',{ keyword: $scope.model.title, source_only: false}).then($response => {
            if(Array.isArray($response)){
                if($response.some($s => $s.title.toLowerCase() == $scope.model.title.toLowerCase())){
                    $scope.flags.isValidTitle = 'times-circle';
                    $scope.flags.isValidTitleMsg = 'Another site has the same title';
                    return;
                }
            }

            $scope.model.domain = $scope.model.title.sanitize();
            $scope.validateDomain();
            $scope.flags.isValidTitle = 'check-circle';
            $scope.flags.isValidTitleMsg = '';
        })
    }

    // Sync page
    $scope.selectSite = function($item){
        $scope.model.source_site = $item.id;
    }
    $scope.isSelectedSite = function($item){
        return $scope.model.source_site == $item.id;
    }

    // Domain
    $scope.validateDomain = function(){

        if(id3IsNullOrEmpty($scope.model.domain)){
            $scope.flags.isValidDomain = 'times-circle';
            $scope.flags.isValidDomainMsg = 'Empty domain';
            return;
        }

        const lDomain = $scope.model.isFullDomain===true
                            ? $scope.model.domain
                            : $scope.model.domain + '.' + $id3PluginContext.networkDomainName;

        if(!lDomain.match(/(\w|-)+\.(\w)+(\.\w+)?/)){
            $scope.flags.isValidDomain = 'times-circle';
            $scope.flags.isValidDomainMsg = 'Not a valid domain';
            return;
        }

        $scope.flags.isValidDomain = 'spinner-third fa-spin';


        $id3Rest.get('aliquando/domain/exists', {name: lDomain}).then($response => {
            console.log('validateDomain', $response);
            if($response === false){
                $scope.flags.isValidDomain = 'check-circle';
                $scope.flags.isValidDomainMsg = '';
            }
            else{
                $scope.flags.isValidDomain = 'times-circle'
                $scope.flags.isValidDomainMsg = 'This domain is already taken';
            }
        });
    }


    // User
    $scope.generatePassword = function(){
        $scope.model.user.password = Math.random().toString(36).slice(2) + '+' + Math.random().toString(36).slice(2,5);
    }

    // Theme page
    $scope.selectTheme = function($item){
        $scope.model.theme = $item.template;
        $scope.model.stylesheet = $item.stylesheet;
        $scope.model.current_theme = $item.name;


        $scope.theme_settings.requirements = [];

        // Check for theme requirement and show wizard page accordingly
        $id3Rest.get('aliquando/third-party/requirements',{theme: $scope.model.stylesheet}).then($response => {
            
            $scope.theme_settings.requirements = $response.map($r => {
                return {path : $r.path}
            });
            $scope.model.requirements = $response.reduce( ($result, $cursor, $index) => {
                if($cursor.defaults == undefined) return $result;
                return Object.assign($result, $cursor.defaults);
            },{});

            // if(id3IsNullOrEmpty($scope.settings.requirements)){
            //     console.log('requirements is empty, so remove the page');
            //     //$scope.unregisterPages(['requirements']);
            // }
            // else{
            //     $scope.registerPageAt({ key: 'requirements',      path: 'new-site/requirements'}, 3);
                
            // }
        })

    }
    $scope.isSelectedTheme = function($item){
        return $scope.model.stylesheet == $item.stylesheet;
    }

    // Social page
    $scope.addSocial = function(){
        if($scope.model.social_medias.some($sm => $sm.type == $scope.newSocialModel.type)) return;

        $scope.model.social_medias.push(angular.copy($scope.newSocialModel))
        $scope.resetSocialModel();
    }

    $scope.removeSocial = function($index){
        $scope.model.social_medias.splice($index,1);
    }

    $scope.resetSocialModel = function(){
        const lMedia = $scope.model.social_medias[$scope.model.social_medias.length - 1];
        const lTypeIndex = lMedia == undefined ? -1 : $scope.socialMedias.indexOf(lMedia.type);
        const lUrl = lMedia == undefined ? '' : lMedia.url;

        let lType = $scope.socialMedias[lTypeIndex + 1];
        $scope.newSocialModel = {
            type: lType,
            url: lUrl
        }
    }
})

id3Plugin
.controller('setupWizardCtrl',
function($scope, $rootScope,$q,$timeout,$id3Settings,$id3Lists, $id3Rest){

    $scope.model = {
        structure_preset: null,
        keep_pages: false,
        office_address: '',
        office_email: '',
        office_phone: '',
        social_medias: []
    };
    $scope.structurePresets = [
        {title: 'Empty', description: 'Leave the structure as it is', selected:true},
        {title: 'Basic', description: 'Add basic pages. ', pages: [
            {title: 'Home', content:'', is_homepage:true},
            {title: 'About us', content:''},
            {title: 'Contact', content:''}
        ]},
        {title: 'Real estate - Agent', description: 'Add real estate pages for an agent', pages: [
            {title: 'Home', content:'[si alias="listings-featured"]', is_homepage:true},
            {title: 'Listings', content: '[si alias="listings"]', children: [
                {title: 'Listing - details', content: '[si_listing]'}
            ]},
            {title: 'About me', content:''},
            {title: 'Contact', content:''}
        ]},
        {title: 'Real estate - Team', description: 'Add real estate pages for a team', pages: [
            {title: 'Home', content:'[si alias="listings-featured"] [si alias="brokers-featured"]', is_homepage:true},
            {title: 'Listings', content: '[si alias="listings"]', children: [
                {title: 'Listing - details', content: '[si_listing]'}
            ]},
            {title: 'Our team', content: '[si alias="brokers"]', children: [
                {title: 'Agent', content: '[si_broker]'}
            ]},
            {title: 'About us', content:''},
            {title: 'Contact', content:''}
        ]},
        {title: 'Real estate - Agency', description: 'Add real estate pages for an agency', pages: [
            {title: 'Home', content:'[si alias="listings-featured"] [si alias="brokers-featured"]', is_homepage:true},
            {title: 'Listings', content: '[si alias="listings"]', children: [
                {title: 'Listing - Details', content: '[si_listing]'}
            ]},
            {title: 'Our team', content: '[si alias="brokers"]', children: [
                {title: 'Agent', content: '[si_broker]'}
            ]},
            {title: 'Offices', content: '[si alias="offices"]', children: [
                {title: 'Office - Details', content: '[si_office]'}
            ]},
            {title: 'About us', content:''},
            {title: 'Contact', content:''}
        ]},
        {title: 'Dental clinic', description: 'Add dental clinic pages', pages: [
            {title: 'Home', content:'', is_homepage:true},
            {title: 'Testimonials', content:''},
            {title: 'Services', content: ''},
            {title: 'Our team', content: ''},
            {title: 'About us', content:''},
            {title: 'Contact', content:''}
        ]}
        
    ];

    $scope.socialMedias = $id3Lists.get('socialMedias');

    $scope.init = function(){
        $scope.registerPages([
            { key: 'structure',         path: 'setup/structure' },
            { key: 'office',            path: 'setup/office' },
            { key: 'social-medias',     path: 'setup/social-medias' },
            
        ]);   

        $scope.registerAction('buildResult', $scope.buildResult);
        $scope.registerValidation('office', _ => $scope.validatePage('office'));

        $scope.resetSocialModel();
    }

    $scope.buildResult = function(){
        return $q( ($resolve,$reject) => {
            $scope.model.structure_preset = $scope.structurePresets.find($p => $p.selected);
            return $resolve($scope.model,null);
        });
    }

    $scope.validatePage = function($key){
        const lHandler = {
            'office' : _ => {
                
                const lResult = !id3IsNullOrEmpty($scope.model.office_address)
                        && !id3IsNullOrEmpty($scope.model.office_email)
                        && !id3IsNullOrEmpty($scope.model.office_phone)

                return lResult;
            }
        };

        console.log('validation for page', $key);
        if(lHandler[$key] == undefined) return true;
        return lHandler[$key]();
    }

    // Preset

    $scope.selectPreset = function($item){
        $scope.structurePresets.forEach($e => {delete $e.selected;});
        $item.selected = true;
    }

    $scope.getPresetPageCount = function($item){
        if($item.pages == undefined) return 0;
        return $item.pages.reduce(($result, $p) => $result+1 + (($p.children!=undefined) ? $p.children.length:0),0);
    }


    // Social page
    $scope.addSocial = function(){
        if($scope.model.social_medias.some($sm => $sm.type == $scope.newSocialModel.type)) return;

        $scope.model.social_medias.push(angular.copy($scope.newSocialModel))
        $scope.resetSocialModel();
    }

    $scope.removeSocial = function($index){
        $scope.model.social_medias.splice($index,1);
    }

    $scope.resetSocialModel = function(){
        if($scope.model.social_medias.length == 0){
            $scope.newSocialModel = {
                type: 'facebook',
                url: ''
            }
        }
        else{
            const lMedia = $scope.model.social_medias[$scope.model.social_medias.length - 1];
            const lTypeIndex = $scope.socialMedias.findIndex(($sm) => $sm.key == lMedia.type);
            const lUrl = lMedia.url;
    
            let lType = $scope.socialMedias[lTypeIndex + 1].key;
            $scope.newSocialModel = {
                type: lType,
                url: lUrl
            }
        }
        
    }
})